/* $Id: is-tomb.h 7200 2024-01-09 17:50:36Z aron $ */

#ifndef WELLTYPE_IS_TOMB_H
#define WELLTYPE_IS_TOMB_H

#include "config.h"
#include "wtruntime.h"

/* instruction set: Tomb */

#define WT_TOMB_INSTRBITS           16
#define WT_TOMB_OPCODEBITS          8
#define WT_TOMB_OPDATABITS          8
#define WT_TOMB_OPCODEMASK          ((1 << WT_TOMB_OPCODEBITS) - 1)
#define WT_TOMB_OPDATAMASK          ((1 << WT_TOMB_OPDATABITS) - 1)
#define WT_TOMB_HIBITIMM            (1 << (WT_TOMB_OPDATABITS - 1))
#define WT_TOMB_HIBITLONGIMM        (1 << (WT_TOMB_OPDATABITS + WT_TOMB_INSTRBITS - 1))
#define WT_TOMB_MASKLONGIMM         ((1 << (WT_TOMB_OPDATABITS + WT_TOMB_INSTRBITS)) - 1)
#define WT_TOMB_GETOPCODE(_instr_)  ((_instr_) >> (WT_TOMB_INSTRBITS - WT_TOMB_OPDATABITS))
#define WT_TOMB_GETOPDATA(_instr_)  ((_instr_) & WT_TOMB_OPDATAMASK)
#define WT_TOMB_BUILDINSTR(_op,_im) ((((_op) & WT_TOMB_OPCODEMASK) << WT_TOMB_OPDATABITS) | ((_im) & WT_TOMB_OPDATAMASK))

#define WT_TOMB_OPCODE(b0,b1,b2,b3,b4,b5,b6,b7) \
	(((b0)<<7) | ((b1)<<6) | ((b2)<<5) | ((b3)<<4) | \
	 ((b4)<<3) | ((b5)<<2) | ((b6)<<1) |  (b7))

enum _wtr_tomb_instructionset
{
	WT_TOMB_OP_NOP      = WT_TOMB_OPCODE(0,0,0,0,0,0,0,0),
/*	                      WT_TOMB_OPCODE(0,0,0,0,0,0,0,1), */
/*	                      WT_TOMB_OPCODE(0,0,0,0,0,0,1,0), */
/*	                      WT_TOMB_OPCODE(0,0,0,0,0,0,1,1), */
/*	                      WT_TOMB_OPCODE(0,0,0,0,0,1,0,0), */
/*	                      WT_TOMB_OPCODE(0,0,0,0,0,1,0,1), */
/*	           LOCK     = WT_TOMB_OPCODE(0,0,0,0,0,1,1,0), */
/*	           ULOCK    = WT_TOMB_OPCODE(0,0,0,0,0,1,1,1), */
	WT_TOMB_OP_LD_I     = WT_TOMB_OPCODE(0,0,0,0,1,0,0,0),
	WT_TOMB_OP_LD_L     = WT_TOMB_OPCODE(0,0,0,0,1,0,0,1),
	WT_TOMB_OP_LD_S     = WT_TOMB_OPCODE(0,0,0,0,1,0,1,0),
/*	                      WT_TOMB_OPCODE(0,0,0,0,1,0,1,1), */
	WT_TOMB_OP_ST_I     = WT_TOMB_OPCODE(0,0,0,0,1,1,0,0),
	WT_TOMB_OP_ST_L     = WT_TOMB_OPCODE(0,0,0,0,1,1,0,1),
	WT_TOMB_OP_ST_S     = WT_TOMB_OPCODE(0,0,0,0,1,1,1,0),
/*	                      WT_TOMB_OPCODE(0,0,0,0,1,1,1,1), */
	WT_TOMB_OP_IDRD_I   = WT_TOMB_OPCODE(0,0,0,1,0,0,0,0),
	WT_TOMB_OP_IDRD_L   = WT_TOMB_OPCODE(0,0,0,1,0,0,0,1),
	WT_TOMB_OP_IDRD_S   = WT_TOMB_OPCODE(0,0,0,1,0,0,1,0),
/*	                      WT_TOMB_OPCODE(0,0,0,1,0,0,1,1), */
	WT_TOMB_OP_PBA_I    = WT_TOMB_OPCODE(0,0,0,1,0,1,0,0),
	WT_TOMB_OP_PBA_L    = WT_TOMB_OPCODE(0,0,0,1,0,1,0,1),
	WT_TOMB_OP_PBA_S    = WT_TOMB_OPCODE(0,0,0,1,0,1,1,0),
/*	                      WT_TOMB_OPCODE(0,0,0,1,0,1,1,1), */
	WT_TOMB_OP_PVT_I    = WT_TOMB_OPCODE(0,0,0,1,1,0,0,0),
	WT_TOMB_OP_PVT_L    = WT_TOMB_OPCODE(0,0,0,1,1,0,0,1),
	WT_TOMB_OP_PVT_S    = WT_TOMB_OPCODE(0,0,0,1,1,0,1,0),
	WT_TOMB_OP_PPC      = WT_TOMB_OPCODE(0,0,0,1,1,0,1,1),
	WT_TOMB_OP_DUP_I    = WT_TOMB_OPCODE(0,0,0,1,1,1,0,0),
	WT_TOMB_OP_DUP_L    = WT_TOMB_OPCODE(0,0,0,1,1,1,0,1),
	WT_TOMB_OP_POP_I    = WT_TOMB_OPCODE(0,0,0,1,1,1,1,0),
	WT_TOMB_OP_POP_L    = WT_TOMB_OPCODE(0,0,0,1,1,1,1,1),
	WT_TOMB_OP_PWZ_I    = WT_TOMB_OPCODE(0,0,1,0,0,0,0,0),
	WT_TOMB_OP_PWZ_L    = WT_TOMB_OPCODE(0,0,1,0,0,0,0,1),
	WT_TOMB_OP_PWO_I    = WT_TOMB_OPCODE(0,0,1,0,0,0,1,0),
	WT_TOMB_OP_PWO_L    = WT_TOMB_OPCODE(0,0,1,0,0,0,1,1),
	WT_TOMB_OP_PWZL_I   = WT_TOMB_OPCODE(0,0,1,0,0,1,0,0),
	WT_TOMB_OP_PWZL_L   = WT_TOMB_OPCODE(0,0,1,0,0,1,0,1),
	WT_TOMB_OP_PWOL_I   = WT_TOMB_OPCODE(0,0,1,0,0,1,1,0),
	WT_TOMB_OP_PWOL_L   = WT_TOMB_OPCODE(0,0,1,0,0,1,1,1),
	WT_TOMB_OP_DVTA_I   = WT_TOMB_OPCODE(0,0,1,0,1,0,0,0),
	WT_TOMB_OP_DVTA_L   = WT_TOMB_OPCODE(0,0,1,0,1,0,0,1),
	WT_TOMB_OP_DVTA_S   = WT_TOMB_OPCODE(0,0,1,0,1,0,1,0),
	WT_TOMB_OP_DVTF     = WT_TOMB_OPCODE(0,0,1,0,1,0,1,1),
	WT_TOMB_OP_OSA_I    = WT_TOMB_OPCODE(0,0,1,0,1,1,0,0),
	WT_TOMB_OP_OSA_L    = WT_TOMB_OPCODE(0,0,1,0,1,1,0,1),
	WT_TOMB_OP_OSA_S    = WT_TOMB_OPCODE(0,0,1,0,1,1,1,0),
	WT_TOMB_OP_OSF      = WT_TOMB_OPCODE(0,0,1,0,1,1,1,1),
	WT_TOMB_OP_OSPU     = WT_TOMB_OPCODE(0,0,1,1,0,0,0,0),
	WT_TOMB_OP_OSPO     = WT_TOMB_OPCODE(0,0,1,1,0,0,0,1),
	WT_TOMB_OP_OSSP     = WT_TOMB_OPCODE(0,0,1,1,0,0,1,0),
	WT_TOMB_OP_TSZ      = WT_TOMB_OPCODE(0,0,1,1,0,0,1,1),
	WT_TOMB_OP_TPU      = WT_TOMB_OPCODE(0,0,1,1,0,1,0,0),
	WT_TOMB_OP_TPO      = WT_TOMB_OPCODE(0,0,1,1,0,1,0,1),
	WT_TOMB_OP_TDR      = WT_TOMB_OPCODE(0,0,1,1,0,1,1,0),
	WT_TOMB_OP_TCL      = WT_TOMB_OPCODE(0,0,1,1,0,1,1,1),
	WT_TOMB_OP_PDSP_I   = WT_TOMB_OPCODE(0,0,1,1,1,0,0,0),
	WT_TOMB_OP_PDSP_L   = WT_TOMB_OPCODE(0,0,1,1,1,0,0,1),
	WT_TOMB_OP_VAS_I    = WT_TOMB_OPCODE(0,0,1,1,1,0,1,0),
	WT_TOMB_OP_VAS_L    = WT_TOMB_OPCODE(0,0,1,1,1,0,1,1),
	WT_TOMB_OP_ENTER_I  = WT_TOMB_OPCODE(0,0,1,1,1,1,0,0),
	WT_TOMB_OP_ENTER_L  = WT_TOMB_OPCODE(0,0,1,1,1,1,0,1),
	WT_TOMB_OP_ENTER_S  = WT_TOMB_OPCODE(0,0,1,1,1,1,1,0),
	WT_TOMB_OP_LEAVE    = WT_TOMB_OPCODE(0,0,1,1,1,1,1,1),
	WT_TOMB_OP_JMP_I    = WT_TOMB_OPCODE(0,1,0,0,0,0,0,0),
	WT_TOMB_OP_JMP_L    = WT_TOMB_OPCODE(0,1,0,0,0,0,0,1),
	WT_TOMB_OP_JMP_S    = WT_TOMB_OPCODE(0,1,0,0,0,0,1,0),
/*	                      WT_TOMB_OPCODE(0,1,0,0,0,0,1,1), */
	WT_TOMB_OP_JT_I     = WT_TOMB_OPCODE(0,1,0,0,0,1,0,0),
	WT_TOMB_OP_JT_L     = WT_TOMB_OPCODE(0,1,0,0,0,1,0,1),
	WT_TOMB_OP_JT_S     = WT_TOMB_OPCODE(0,1,0,0,0,1,1,0),
/*	                      WT_TOMB_OPCODE(0,1,0,0,0,1,1,1), */
	WT_TOMB_OP_JF_I     = WT_TOMB_OPCODE(0,1,0,0,1,0,0,0),
	WT_TOMB_OP_JF_L     = WT_TOMB_OPCODE(0,1,0,0,1,0,0,1),
	WT_TOMB_OP_JF_S     = WT_TOMB_OPCODE(0,1,0,0,1,0,1,0),
/*	                      WT_TOMB_OPCODE(0,1,0,0,1,0,1,1), */
	WT_TOMB_OP_JA_I     = WT_TOMB_OPCODE(0,1,0,0,1,1,0,0),
	WT_TOMB_OP_JA_L     = WT_TOMB_OPCODE(0,1,0,0,1,1,0,1),
	WT_TOMB_OP_JA_S     = WT_TOMB_OPCODE(0,1,0,0,1,1,1,0),
/*	                      WT_TOMB_OPCODE(0,1,0,0,1,1,1,1), */
	WT_TOMB_OP_EXEC_I   = WT_TOMB_OPCODE(0,1,0,1,0,0,0,0),
	WT_TOMB_OP_EXEC_L   = WT_TOMB_OPCODE(0,1,0,1,0,0,0,1),
	WT_TOMB_OP_EXEC_S   = WT_TOMB_OPCODE(0,1,0,1,0,0,1,0),
/*	                      WT_TOMB_OPCODE(0,1,0,1,0,0,1,1), */
	WT_TOMB_OP_DLE_I    = WT_TOMB_OPCODE(0,1,0,1,0,1,0,0),
	WT_TOMB_OP_DLE_L    = WT_TOMB_OPCODE(0,1,0,1,0,1,0,1),
	WT_TOMB_OP_DLE_S    = WT_TOMB_OPCODE(0,1,0,1,0,1,1,0),
	WT_TOMB_OP_CFP      = WT_TOMB_OPCODE(0,1,0,1,0,1,1,1),
	WT_TOMB_OP_MKFP_I   = WT_TOMB_OPCODE(0,1,0,1,1,0,0,0),
	WT_TOMB_OP_MKFP_L   = WT_TOMB_OPCODE(0,1,0,1,1,0,0,1),
	WT_TOMB_OP_MKFP_S   = WT_TOMB_OPCODE(0,1,0,1,1,0,1,0),
	WT_TOMB_OP_TTD      = WT_TOMB_OPCODE(0,1,0,1,1,0,1,1), /* since v3.0.0.2 */
	WT_TOMB_OP_LDFP     = WT_TOMB_OPCODE(0,1,0,1,1,1,0,0),
	WT_TOMB_OP_STFP     = WT_TOMB_OPCODE(0,1,0,1,1,1,0,1),
	WT_TOMB_OP_FPS_I    = WT_TOMB_OPCODE(0,1,0,1,1,1,1,0),
	WT_TOMB_OP_FPS_L    = WT_TOMB_OPCODE(0,1,0,1,1,1,1,1),
	WT_TOMB_OP_STRN     = WT_TOMB_OPCODE(0,1,1,0,0,0,0,0),
	WT_TOMB_OP_STRC     = WT_TOMB_OPCODE(0,1,1,0,0,0,0,1),
	WT_TOMB_OP_STRL     = WT_TOMB_OPCODE(0,1,1,0,0,0,1,0),
	WT_TOMB_OP_STRD     = WT_TOMB_OPCODE(0,1,1,0,0,0,1,1),
	WT_TOMB_OP_STRCR_I  = WT_TOMB_OPCODE(0,1,1,0,0,1,0,0),
	WT_TOMB_OP_STRCR_L  = WT_TOMB_OPCODE(0,1,1,0,0,1,0,1),
	WT_TOMB_OP_STRCR_S  = WT_TOMB_OPCODE(0,1,1,0,0,1,1,0),
	WT_TOMB_OP_SEQN_I   = WT_TOMB_OPCODE(0,1,1,0,0,1,1,1),
	WT_TOMB_OP_SEQN_L   = WT_TOMB_OPCODE(0,1,1,0,1,0,0,0),
	WT_TOMB_OP_SEQN_S   = WT_TOMB_OPCODE(0,1,1,0,1,0,0,1),
	WT_TOMB_OP_SEQD     = WT_TOMB_OPCODE(0,1,1,0,1,0,1,0),
	WT_TOMB_OP_SEQIR_I  = WT_TOMB_OPCODE(0,1,1,0,1,0,1,1),
	WT_TOMB_OP_SEQIR_L  = WT_TOMB_OPCODE(0,1,1,0,1,1,0,0),
	WT_TOMB_OP_SEQIR_S  = WT_TOMB_OPCODE(0,1,1,0,1,1,0,1),
	WT_TOMB_OP_SEQL     = WT_TOMB_OPCODE(0,1,1,0,1,1,1,0),
	WT_TOMB_OP_RECN_I   = WT_TOMB_OPCODE(0,1,1,0,1,1,1,1),
	WT_TOMB_OP_RECN_L   = WT_TOMB_OPCODE(0,1,1,1,0,0,0,0),
	WT_TOMB_OP_RECN_S   = WT_TOMB_OPCODE(0,1,1,1,0,0,0,1),
	WT_TOMB_OP_RECD     = WT_TOMB_OPCODE(0,1,1,1,0,0,1,0),
	WT_TOMB_OP_RECFR_I  = WT_TOMB_OPCODE(0,1,1,1,0,0,1,1),
	WT_TOMB_OP_RECFR_L  = WT_TOMB_OPCODE(0,1,1,1,0,1,0,0),
	WT_TOMB_OP_RECFR_S  = WT_TOMB_OPCODE(0,1,1,1,0,1,0,1),
/*	                      WT_TOMB_OPCODE(0,1,1,1,0,1,1,0), */
/*	                      WT_TOMB_OPCODE(0,1,1,1,0,1,1,1), */
/*	                      WT_TOMB_OPCODE(0,1,1,1,1,0,0,0), */
/*	                      WT_TOMB_OPCODE(0,1,1,1,1,0,0,1), */
/*	                      WT_TOMB_OPCODE(0,1,1,1,1,0,1,0), */
	WT_TOMB_OP_ERAISE   = WT_TOMB_OPCODE(0,1,1,1,1,0,1,1),
	WT_TOMB_OP_EXHAN_I  = WT_TOMB_OPCODE(0,1,1,1,1,1,0,0),
	WT_TOMB_OP_EXHAN_L  = WT_TOMB_OPCODE(0,1,1,1,1,1,0,1),
	WT_TOMB_OP_EXHAN_S  = WT_TOMB_OPCODE(0,1,1,1,1,1,1,0),
	WT_TOMB_OP_EHCL     = WT_TOMB_OPCODE(0,1,1,1,1,1,1,1),
	WT_TOMB_OP_ADD_I    = WT_TOMB_OPCODE(1,0,0,0,0,0,0,0),
	WT_TOMB_OP_ADD_S    = WT_TOMB_OPCODE(1,0,0,0,0,0,0,1),
	WT_TOMB_OP_SUB_I    = WT_TOMB_OPCODE(1,0,0,0,0,0,1,0),
	WT_TOMB_OP_SUB_S    = WT_TOMB_OPCODE(1,0,0,0,0,0,1,1),
	WT_TOMB_OP_MUL_I    = WT_TOMB_OPCODE(1,0,0,0,0,1,0,0),
	WT_TOMB_OP_MUL_S    = WT_TOMB_OPCODE(1,0,0,0,0,1,0,1),
	WT_TOMB_OP_DIV_I    = WT_TOMB_OPCODE(1,0,0,0,0,1,1,0),
	WT_TOMB_OP_DIV_S    = WT_TOMB_OPCODE(1,0,0,0,0,1,1,1),
	WT_TOMB_OP_SHL_I    = WT_TOMB_OPCODE(1,0,0,0,1,0,0,0),
	WT_TOMB_OP_SHL_S    = WT_TOMB_OPCODE(1,0,0,0,1,0,0,1),
	WT_TOMB_OP_SHR_I    = WT_TOMB_OPCODE(1,0,0,0,1,0,1,0),
	WT_TOMB_OP_SHR_S    = WT_TOMB_OPCODE(1,0,0,0,1,0,1,1),
	WT_TOMB_OP_SAR_I    = WT_TOMB_OPCODE(1,0,0,0,1,1,0,0),
	WT_TOMB_OP_SAR_S    = WT_TOMB_OPCODE(1,0,0,0,1,1,0,1),
	WT_TOMB_OP_ROR_I    = WT_TOMB_OPCODE(1,0,0,0,1,1,1,0),
	WT_TOMB_OP_ROR_S    = WT_TOMB_OPCODE(1,0,0,0,1,1,1,1),
	WT_TOMB_OP_SMUL_I   = WT_TOMB_OPCODE(1,0,0,1,0,0,0,0),
	WT_TOMB_OP_SMUL_S   = WT_TOMB_OPCODE(1,0,0,1,0,0,0,1),
	WT_TOMB_OP_SDIV_I   = WT_TOMB_OPCODE(1,0,0,1,0,0,1,0),
	WT_TOMB_OP_SDIV_S   = WT_TOMB_OPCODE(1,0,0,1,0,0,1,1),
	WT_TOMB_OP_MOD      = WT_TOMB_OPCODE(1,0,0,1,0,1,0,0),
	WT_TOMB_OP_SMOD     = WT_TOMB_OPCODE(1,0,0,1,0,1,0,1),
	WT_TOMB_OP_LSOR_I   = WT_TOMB_OPCODE(1,0,0,1,0,1,1,0),
	WT_TOMB_OP_LSOR_L   = WT_TOMB_OPCODE(1,0,0,1,0,1,1,1),
	WT_TOMB_OP_RSB_I    = WT_TOMB_OPCODE(1,0,0,1,1,0,0,0),
	WT_TOMB_OP_RSB_L    = WT_TOMB_OPCODE(1,0,0,1,1,0,0,1),
	WT_TOMB_OP_ADDL     = WT_TOMB_OPCODE(1,0,0,1,1,0,1,0),
	WT_TOMB_OP_SUBL     = WT_TOMB_OPCODE(1,0,0,1,1,0,1,1),
	WT_TOMB_OP_MULL     = WT_TOMB_OPCODE(1,0,0,1,1,1,0,0),
	WT_TOMB_OP_DIVL     = WT_TOMB_OPCODE(1,0,0,1,1,1,0,1),
	WT_TOMB_OP_SHLL     = WT_TOMB_OPCODE(1,0,0,1,1,1,1,0),
	WT_TOMB_OP_SHRL     = WT_TOMB_OPCODE(1,0,0,1,1,1,1,1),
	WT_TOMB_OP_SARL     = WT_TOMB_OPCODE(1,0,1,0,0,0,0,0),
	WT_TOMB_OP_RORL     = WT_TOMB_OPCODE(1,0,1,0,0,0,0,1),
	WT_TOMB_OP_XOR      = WT_TOMB_OPCODE(1,0,1,0,0,0,1,0),
	WT_TOMB_OP_BAND     = WT_TOMB_OPCODE(1,0,1,0,0,0,1,1),
	WT_TOMB_OP_BOR      = WT_TOMB_OPCODE(1,0,1,0,0,1,0,0),
	WT_TOMB_OP_BNOT     = WT_TOMB_OPCODE(1,0,1,0,0,1,0,1),
	WT_TOMB_OP_XORL     = WT_TOMB_OPCODE(1,0,1,0,0,1,1,0),
	WT_TOMB_OP_BANDL    = WT_TOMB_OPCODE(1,0,1,0,0,1,1,1),
	WT_TOMB_OP_BORL     = WT_TOMB_OPCODE(1,0,1,0,1,0,0,0),
	WT_TOMB_OP_BNOTL    = WT_TOMB_OPCODE(1,0,1,0,1,0,0,1),
	WT_TOMB_OP_LSORL_I  = WT_TOMB_OPCODE(1,0,1,0,1,0,1,0),
	WT_TOMB_OP_LSORL_L  = WT_TOMB_OPCODE(1,0,1,0,1,0,1,1),
	WT_TOMB_OP_SMULL    = WT_TOMB_OPCODE(1,0,1,0,1,1,0,0),
	WT_TOMB_OP_SDIVL    = WT_TOMB_OPCODE(1,0,1,0,1,1,0,1),
	WT_TOMB_OP_MODL     = WT_TOMB_OPCODE(1,0,1,0,1,1,1,0),
	WT_TOMB_OP_SMODL    = WT_TOMB_OPCODE(1,0,1,0,1,1,1,1),
	WT_TOMB_OP_NEG      = WT_TOMB_OPCODE(1,0,1,1,0,0,0,0),
	WT_TOMB_OP_NEGL     = WT_TOMB_OPCODE(1,0,1,1,0,0,0,1),
	WT_TOMB_OP_FADD     = WT_TOMB_OPCODE(1,0,1,1,0,0,1,0),
	WT_TOMB_OP_FSUB     = WT_TOMB_OPCODE(1,0,1,1,0,0,1,1),
	WT_TOMB_OP_FMUL     = WT_TOMB_OPCODE(1,0,1,1,0,1,0,0),
	WT_TOMB_OP_FDIV     = WT_TOMB_OPCODE(1,0,1,1,0,1,0,1),
	WT_TOMB_OP_FADDL    = WT_TOMB_OPCODE(1,0,1,1,0,1,1,0),
	WT_TOMB_OP_FSUBL    = WT_TOMB_OPCODE(1,0,1,1,0,1,1,1),
	WT_TOMB_OP_FMULL    = WT_TOMB_OPCODE(1,0,1,1,1,0,0,0),
	WT_TOMB_OP_FDIVL    = WT_TOMB_OPCODE(1,0,1,1,1,0,0,1),
	WT_TOMB_OP_FNEG     = WT_TOMB_OPCODE(1,0,1,1,1,0,1,0),
	WT_TOMB_OP_FNEGL    = WT_TOMB_OPCODE(1,0,1,1,1,0,1,1),
	WT_TOMB_OP_FMOD     = WT_TOMB_OPCODE(1,0,1,1,1,1,0,0),
	WT_TOMB_OP_FMODL    = WT_TOMB_OPCODE(1,0,1,1,1,1,0,1),
	WT_TOMB_OP_FABS     = WT_TOMB_OPCODE(1,0,1,1,1,1,1,0),
	WT_TOMB_OP_FABSL    = WT_TOMB_OPCODE(1,0,1,1,1,1,1,1),
	WT_TOMB_OP_CMP      = WT_TOMB_OPCODE(1,1,0,0,0,0,0,0), /* result: { -1, 0, 1 } */
	WT_TOMB_OP_SCMP     = WT_TOMB_OPCODE(1,1,0,0,0,0,0,1),
	WT_TOMB_OP_CMPL     = WT_TOMB_OPCODE(1,1,0,0,0,0,1,0),
	WT_TOMB_OP_SCMPL    = WT_TOMB_OPCODE(1,1,0,0,0,0,1,1),
	WT_TOMB_OP_FCMP     = WT_TOMB_OPCODE(1,1,0,0,0,1,0,0),
	WT_TOMB_OP_FCMPL    = WT_TOMB_OPCODE(1,1,0,0,0,1,0,1),
	WT_TOMB_OP_SCMPB    = WT_TOMB_OPCODE(1,1,0,0,0,1,1,0), /* since v3.0.0.3 */
	WT_TOMB_OP_SCMPH    = WT_TOMB_OPCODE(1,1,0,0,0,1,1,1), /* since v3.0.0.3 */
	WT_TOMB_OP_QLT      = WT_TOMB_OPCODE(1,1,0,0,1,0,0,0), /* {-1, 0, 1} -> {0, 1} */
	WT_TOMB_OP_QLE      = WT_TOMB_OPCODE(1,1,0,0,1,0,0,1),
	WT_TOMB_OP_QGT      = WT_TOMB_OPCODE(1,1,0,0,1,0,1,0),
	WT_TOMB_OP_QGE      = WT_TOMB_OPCODE(1,1,0,0,1,0,1,1),
	WT_TOMB_OP_QEQ      = WT_TOMB_OPCODE(1,1,0,0,1,1,0,0),
	WT_TOMB_OP_QNE      = WT_TOMB_OPCODE(1,1,0,0,1,1,0,1),
	WT_TOMB_OP_OSDR     = WT_TOMB_OPCODE(1,1,0,0,1,1,1,0), /* since v3.0.0.1 */
	WT_TOMB_OP_MSKBTM   = WT_TOMB_OPCODE(1,1,0,0,1,1,1,1), /* since v3.0.0.3 */
	WT_TOMB_OP_LNOT     = WT_TOMB_OPCODE(1,1,0,1,0,0,0,0),
	WT_TOMB_OP_LAND     = WT_TOMB_OPCODE(1,1,0,1,0,0,0,1),
	WT_TOMB_OP_LOR      = WT_TOMB_OPCODE(1,1,0,1,0,0,1,0),
	WT_TOMB_OP_LXOR     = WT_TOMB_OPCODE(1,1,0,1,0,0,1,1),
	WT_TOMB_OP_LDB      = WT_TOMB_OPCODE(1,1,0,1,0,1,0,0),
	WT_TOMB_OP_STB      = WT_TOMB_OPCODE(1,1,0,1,0,1,0,1),
	WT_TOMB_OP_LDH      = WT_TOMB_OPCODE(1,1,0,1,0,1,1,0),
	WT_TOMB_OP_STH      = WT_TOMB_OPCODE(1,1,0,1,0,1,1,1),
	WT_TOMB_OP_ADDB_I   = WT_TOMB_OPCODE(1,1,0,1,1,0,0,0), /* since v3.0.0.3 */
	WT_TOMB_OP_ADDB_S   = WT_TOMB_OPCODE(1,1,0,1,1,0,0,1), /* since v3.0.0.3 */
	WT_TOMB_OP_SUBB_I   = WT_TOMB_OPCODE(1,1,0,1,1,0,1,0), /* since v3.0.0.3 */
	WT_TOMB_OP_SUBB_S   = WT_TOMB_OPCODE(1,1,0,1,1,0,1,1), /* since v3.0.0.3 */
	WT_TOMB_OP_MULB     = WT_TOMB_OPCODE(1,1,0,1,1,1,0,0), /* since v3.0.0.3 */
	WT_TOMB_OP_SMULB    = WT_TOMB_OPCODE(1,1,0,1,1,1,0,1), /* since v3.0.0.3 */
	WT_TOMB_OP_DIVB     = WT_TOMB_OPCODE(1,1,0,1,1,1,1,0), /* since v3.0.0.3 */
	WT_TOMB_OP_SDIVB    = WT_TOMB_OPCODE(1,1,0,1,1,1,1,1), /* since v3.0.0.3 */
	WT_TOMB_OP_MODB     = WT_TOMB_OPCODE(1,1,1,0,0,0,0,0), /* since v3.0.0.3 */
	WT_TOMB_OP_SMODB    = WT_TOMB_OPCODE(1,1,1,0,0,0,0,1), /* since v3.0.0.3 */
	WT_TOMB_OP_SHLB_I   = WT_TOMB_OPCODE(1,1,1,0,0,0,1,0), /* since v3.0.0.3 */
	WT_TOMB_OP_SHLB_S   = WT_TOMB_OPCODE(1,1,1,0,0,0,1,1), /* since v3.0.0.3 */
	WT_TOMB_OP_RORB     = WT_TOMB_OPCODE(1,1,1,0,0,1,0,0), /* since v3.0.0.3 */
	WT_TOMB_OP_BNOTB    = WT_TOMB_OPCODE(1,1,1,0,0,1,0,1), /* since v3.0.0.3 */
	WT_TOMB_OP_ADDH_I   = WT_TOMB_OPCODE(1,1,1,0,0,1,1,0), /* since v3.0.0.3 */
	WT_TOMB_OP_ADDH_S   = WT_TOMB_OPCODE(1,1,1,0,0,1,1,1), /* since v3.0.0.3 */
	WT_TOMB_OP_SUBH_I   = WT_TOMB_OPCODE(1,1,1,0,1,0,0,0), /* since v3.0.0.3 */
	WT_TOMB_OP_SUBH_S   = WT_TOMB_OPCODE(1,1,1,0,1,0,0,1), /* since v3.0.0.3 */
	WT_TOMB_OP_MULH     = WT_TOMB_OPCODE(1,1,1,0,1,0,1,0), /* since v3.0.0.3 */
	WT_TOMB_OP_SMULH    = WT_TOMB_OPCODE(1,1,1,0,1,0,1,1), /* since v3.0.0.3 */
	WT_TOMB_OP_DIVH     = WT_TOMB_OPCODE(1,1,1,0,1,1,0,0), /* since v3.0.0.3 */
	WT_TOMB_OP_SDIVH    = WT_TOMB_OPCODE(1,1,1,0,1,1,0,1), /* since v3.0.0.3 */
	WT_TOMB_OP_MODH     = WT_TOMB_OPCODE(1,1,1,0,1,1,1,0), /* since v3.0.0.3 */
	WT_TOMB_OP_SMODH    = WT_TOMB_OPCODE(1,1,1,0,1,1,1,1), /* since v3.0.0.3 */
	WT_TOMB_OP_SHLH_I   = WT_TOMB_OPCODE(1,1,1,1,0,0,0,0), /* since v3.0.0.3 */
	WT_TOMB_OP_SHLH_S   = WT_TOMB_OPCODE(1,1,1,1,0,0,0,1), /* since v3.0.0.3 */
	WT_TOMB_OP_RORH     = WT_TOMB_OPCODE(1,1,1,1,0,0,1,0), /* since v3.0.0.3 */
	WT_TOMB_OP_BNOTH    = WT_TOMB_OPCODE(1,1,1,1,0,0,1,1), /* since v3.0.0.3 */
	WT_TOMB_OP_SGNXD    = WT_TOMB_OPCODE(1,1,1,1,0,1,0,0), /* since v3.0.0.3 */
/*	                      WT_TOMB_OPCODE(1,1,1,1,0,1,0,1), */
/*	                      WT_TOMB_OPCODE(1,1,1,1,0,1,1,0), */
/*	                      WT_TOMB_OPCODE(1,1,1,1,0,1,1,1), */
/*	                      WT_TOMB_OPCODE(1,1,1,1,1,0,0,0), */
/*	                      WT_TOMB_OPCODE(1,1,1,1,1,0,0,1), */
/*	                      WT_TOMB_OPCODE(1,1,1,1,1,0,1,0), */
/*	                      WT_TOMB_OPCODE(1,1,1,1,1,0,1,1), */
/*	                      WT_TOMB_OPCODE(1,1,1,1,1,1,0,0), */
/*	                      WT_TOMB_OPCODE(1,1,1,1,1,1,0,1), */
/*	                      WT_TOMB_OPCODE(1,1,1,1,1,1,1,0), */
/*	                      WT_TOMB_OPCODE(1,1,1,1,1,1,1,1), */
};
typedef enum _wtr_tomb_instructionset wtr_tomb_instructionset;

#endif /* WELLTYPE_IS_TOMB_H */
