/* $Id: ast_basictype.h 7194 2024-01-08 19:29:54Z aron $ */
/*
    Welltype parser and AST
    Copyright (C) 2014-2023  Aron Barath

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef WTPARSER_AST_BASICTYPE_H
#define WTPARSER_AST_BASICTYPE_H

#include "../astnode.h"
#include "ast_type.h"
#include "welltype/wtb3.h"

namespace wtast
{

class BasicType : public Type
{
	std::string mName;
	wt_u32      mTypeId;
	wt_u32      mLoadSize;
	bool        mIsObject;
	bool        mIsDefaultConstructible;

public:
	BasicType(const std::string & name, wt_u32 type_id, wt_u32 loadsize)
		: Type()
		, mName(name)
		, mTypeId(type_id)
		, mLoadSize(loadsize)
		, mIsObject(WTB3T_STRING==type_id || WTB3T_FILE==type_id || WTB3T_DIR==type_id)
		, mIsDefaultConstructible(WTB3T_FILE!=type_id && WTB3T_DIR!=type_id)
	{
		// ok
	}

	~BasicType()
	{
		// ok
	}

	void Accept(AstVisitor* vis);

	BasicType* Copy();

	Kind GetKind() const
	{
		return BASICTYPE;
	}

	std::string GetName() const
	{
		return mName;
	}

	wt_u32 GetTypeId() const
	{
		return mTypeId;
	}

	wt_u32 GetBasicTypeId() const
	{
		return mTypeId & ~WTB3TF_MUTABLE;
	}

	wt_u32 GetLoadSize() const
	{
		return mLoadSize;
	}

	bool IsObject() const
	{
		return mIsObject;
	}

	bool IsDefaultConstructible() const
	{
		return mIsDefaultConstructible;
	}

	void SetMutable()
	{
		mTypeId |= WTB3TF_MUTABLE;
	}
};

} // namespace wtast

#endif // WTPARSER_AST_BASICTYPE_H
