/* $Id: ast_elementaccess.h 6504 2023-04-03 17:46:07Z aron $ */
/*
    Welltype parser and AST
    Copyright (C) 2014-2023  Aron Barath

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef WTPARSER_AST_ELEMENTACCESS_H
#define WTPARSER_AST_ELEMENTACCESS_H

#include "../astnode.h"
#include "ast_lvalue.h"
#include "ast_type.h"
#include "welltype/wtruntime.h"
#include <string>

namespace wtast
{

class Expression;

class ElementAccess : public LValue
{
	LValue*     mLValue;
	Expression* mIndexing;
	TypeInfo*   mResultType;

public:
	ElementAccess(LValue* lvalue, Expression* indexing, TypeInfo* result_type)
		: LValue()
		, mLValue(lvalue)
		, mIndexing(indexing)
		, mResultType(result_type)
	{
		// ok
	}

	~ElementAccess();

	void Accept(AstVisitor* vis);

	ElementAccess* Copy();

	Kind GetKind() const
	{
		return ELEMENTACCESS;
	}

	LValue* GetLValue()
	{
		return mLValue;
	}

	bool OriginateFrom(Variable::Class varclass)
	{
		return mLValue->OriginateFrom(varclass);
	}

	Expression* GetIndexing()
	{
		return mIndexing;
	}

	TypeInfo* GetType()
	{
		return mResultType;
	}
};

} // namespace wtast

#endif // WTPARSER_AST_ELEMENTACCESS_H
