/* $Id: ast_fieldassign.h 6504 2023-04-03 17:46:07Z aron $ */
/*
    Welltype parser and AST
    Copyright (C) 2014-2023  Aron Barath

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef WTPARSER_AST_FIELDASSIGN_H
#define WTPARSER_AST_FIELDASSIGN_H

#include <string>
#include <deque>
#include "../astnode.h"
#include "welltype/wtruntime.h"

namespace wtast
{

class Expression;

class FieldAssign : public AstNode
{
	typedef std::deque<std::string> names_t;

	names_t     mNames;
	Expression* mExpr;

public:
	FieldAssign(Expression* expr)
		: AstNode()
		, mExpr(expr)
	{
		// ok
	}

	~FieldAssign();

	void Accept(AstVisitor* vis);

	FieldAssign* Copy();

	Kind GetKind() const
	{
		return FIELDASSIGN;
	}

	void AddFieldNameFront(const std::string & name)
	{
		mNames.push_front(name);
	}

	void AddFieldNameBack(const std::string & name)
	{
		mNames.push_back(name);
	}

	std::size_t GetNumberOfFieldNames() const
	{
		return mNames.size();
	}

	std::string & GetFieldName(std::size_t idx)
	{
		return mNames[idx];
	}

	Expression* GetExpression()
	{
		return mExpr;
	}
};

} // namespace wtast

#endif // WTPARSER_AST_FIELDASSIGN_H
