/* $Id: ast_forstatement.h 6310 2023-01-16 20:39:48Z aron $ */
/*
    Welltype parser and AST
    Copyright (C) 2014-2023  Aron Barath

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef WTPARSER_AST_FORSTATEMENT_H
#define WTPARSER_AST_FORSTATEMENT_H

#include "../astnode.h"
#include "ast_statement.h"

namespace wtast
{

class Assignment;
class Expression;
class Compound;

class ForStatement : public Statement
{
	Assignment* mInit;
	Expression* mCond;
	Assignment* mStep;
	Compound*   mBody;

public:
	ForStatement(Assignment* init, Expression* cond, Assignment* step, Compound* body)
		: Statement()
		, mInit(init)
		, mCond(cond)
		, mStep(step)
		, mBody(body)
	{
		// ok
	}

	~ForStatement();

	void Accept(AstVisitor* vis);

	ForStatement* Copy();

	Kind GetKind() const
	{
		return FORSTATEMENT;
	}

	Assignment* GetInit()
	{
		return mInit;
	}

	Expression* GetCondition()
	{
		return mCond;
	}

	Assignment* GetStep()
	{
		return mStep;
	}

	Compound* GetBody()
	{
		return mBody;
	}
};

} // namespace wtast

#endif // WTPARSER_AST_FORSTATEMENT_H
