/* $Id: ast_recordtype.cpp 6504 2023-04-03 17:46:07Z aron $ */
/*
    Welltype parser and AST
    Copyright (C) 2014-2023  Aron Barath

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "ast_recordtype.h"
#include "ast_recordfield.h"
#include "../astvisitor.h"
#include "../../wtruntime/types.h"

wtast::RecordType::~RecordType()
{
	while(!mFields.empty())
	{
		delete mFields.front();
		mFields.pop_front();
	}
}

bool wtast::RecordType::IsDefaultConstructible() const
{
	if(mLimited && mRestricted)
	{
		return false;
	}

	for(fields_t::const_iterator it=mFields.begin(),end=mFields.end();it!=end;++it)
	{
		if(!const_cast<wtast::RecordField*>(*it)->GetType()->IsDefaultConstructible())
		{
			return false;
		}
	}

	return true;
}

bool wtast::RecordType::HasFunctionType() const
{
	for(fields_t::const_iterator it=mFields.begin(),end=mFields.end();it!=end;++it)
	{
		if(const_cast<wtast::RecordField*>(*it)->GetType()->HasFunctionType())
		{
			return true;
		}
	}

	return false;
}

bool wtast::RecordType::IsCopyable() const
{
	if(mLimited && mRestricted)
	{
		return false;
	}

	for(fields_t::const_iterator it=mFields.begin(),end=mFields.end();it!=end;++it)
	{
		if(!const_cast<wtast::RecordField*>(*it)->GetType()->IsCopyable())
		{
			return false;
		}
	}

	return true;
}

void wtast::RecordType::Accept(AstVisitor* vis)
{
	if(vis->Visit(this))
	{
		for(fields_t::iterator it=mFields.begin(),end=mFields.end();it!=end;++it)
		{
			(*it)->Accept(vis);
		}

		vis->EndVisit(this);
	}
}

wtast::RecordType* wtast::RecordType::Copy()
{
	RecordType* rec = new RecordType(mName, mTypeId, mPrivate, mLimited, mRestricted);

	for(fields_t::iterator it=mFields.begin(),end=mFields.end();it!=end;++it)
	{
		rec->AddField((*it)->Copy());
	}

	return rec;
}

wt_u32 wtast::RecordType::GetLoadSize() const
{
	return sizeof(wt_vmptr);
}

bool wtast::RecordType::AddField(wtast::RecordField* field)
{
	if(mFieldMap.find(field->GetName())==mFieldMap.end())
	{
		field->SetOffset(mTotalSize);
		mTotalSize += field->GetType()->GetLoadSize();

		mFields.push_back(field);
		mFieldMap[field->GetName()] = field;

		return true;
	}

	return false;
}
