/* $Id: ast_simultaneousexpression.h 6504 2023-04-03 17:46:07Z aron $ */
/*
    Welltype parser and AST
    Copyright (C) 2014-2023  Aron Barath

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef WTPARSER_AST_SIMULTANEOUSEXPRESSION_H
#define WTPARSER_AST_SIMULTANEOUSEXPRESSION_H

#include "../astnode.h"
#include "ast_expression.h"
#include "welltype/wtruntime.h"
#include <deque>
#include <string>

namespace wtparser
{
class ParserHelper;
} // wtparser

namespace wtast
{

class SimultaneousType;

class SimultaneousExpression : public Expression
{
	typedef std::deque<Expression*> expressions_t;
	expressions_t     mExpressions;
	SimultaneousType* mTypeInfo;

public:
	SimultaneousExpression(Expression* expr1=0, Expression* expr2=0);

	~SimultaneousExpression();

	void Accept(AstVisitor* vis);

	SimultaneousExpression* Copy();

	Kind GetKind() const
	{
		return SIMULTANEOUSEXPRESSION;
	}

	bool OriginateFrom(Variable::Class varclass);

	void AddExpression(Expression* expr);

	virtual std::size_t GetNumberOfExpressions() const // from wtast::Expression
	{
		return mExpressions.size();
	}

	virtual Expression* GetExpression(std::size_t idx) // from wtast::Expression
	{
		if(idx<mExpressions.size())
		{
			return mExpressions[idx];
		}
		else
		{
			return 0;
		}
	}

	virtual TypeInfo* GetType(); // from wtast::Expression

	virtual bool IsSimultaneous() const { return true; } // from wtast::Expression
	virtual bool IsTemporary() const; // from wtast::Expression

	friend class wtparser::ParserHelper;
};

} // namespace wtast

#endif // WTPARSER_AST_SIMULTANEOUSEXPRESSION_H
