#ifndef WTRUNTIME_HASH_H
#define WTRUNTIME_HASH_H

#include "config.h"
#include "types.h"
#include "byteorder.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


struct wtr_lreader;


/* Find hash algorithm by name in 'algs' (it must be terminated by */
/* name==NULL; accepts algs==NULL as well. */
const struct wtr_hashalg* wtr_hash_find_by_name(
	const struct wtr_hashalg* algs, const char* const name);


/* Find hash algorithm by ID in 'algs' (it must be terminated by */
/* name==NULL; accepts algs==NULL as well. */
const struct wtr_hashalg* wtr_hash_find_by_id(
	const struct wtr_hashalg* algs, const wt_u16 id);


/* Calculate the hash of a Welltype Binary. */
/* Returns: error code */
/*   out_digest - the digest will be stored in this buffer (the caller must ensure that the digest fits in the buffer) */
/*   alg        - hash algorithm */
/*   data       - the Welltype Binary data (must start with the WTB signature) */
/*   size       - length of the 'data' in octets */
wt_error wtr_hash_calc(void* out_digest, const struct wtr_hashalg* const alg,
	const void* data, size_t size, wt_bo bo);


/* same as wtr_hash_calc(), but uses a loader-reader */
wt_error wtr_hash_calc_lr(void* out_digest, struct wtr_lreader* const lr,
	const struct wtr_hashalg* const alg);


/* Checks whether the computed hash matches with the reference hash value. */
/* Returns: error code */
/*   digest - the reference hash value */
/*   alg    - hash algorithm */
/*   data   - the Welltype Binary data (must start with the WTB signature) */
/*   size   - length of the 'data' in octets */
wt_error wtr_hash_check(const void* digest,
	const struct wtr_hashalg* const alg, const void* data, size_t size,
	wt_bo bo);

/* same as wtr_hash_check(), but uses a loader-reader */
wt_error wtr_hash_check_lr(struct wtr_lreader* const lr,
	const struct wtr_hashalg* const alg, const wt_uint32_t digest_offs);


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* WTRUNTIME_HASH_H */
