#include "config_private.h"
#include "program.h"
#include "types.h"
#include "vm.h"
#include "symtab.h"
#include <string.h>

wt_error wtrProgramSetUserPointer(wt_program program, void* ptr)
{
	program->user_ptr = ptr;
	return WTE_OK;
}

wt_error wtrProgramGetUserPointer(wt_program program, void** pptr)
{
	*pptr = program->user_ptr;
	return WTE_OK;
}

const char* wtrProgramGetMetadata(wt_program program, const char* const key)
{
	const size_t num = wtr_vector_size(&program->metadata_vec);

	if(0<num)
	{
		struct wtr_metadata** ptr = (struct wtr_metadata**)
			wtr_vector_at(&program->metadata_vec, 0);

		struct wtr_metadata** const end = ptr + num;

		while(ptr<end)
		{
			struct wtr_metadata* const metadata = *ptr++;

			if(0==strcmp(key, metadata->key))
			{
				return metadata->val;
			}
		}
	}

	return NULL;
}

void wtr_program_unref_symtabs(struct wtr_program* const prog)
{
	const size_t size = wtr_vector_size(&prog->referenced_symtabs);

	if(0<size)
	{
		struct wtr_symtab** p_symtab =
			(struct wtr_symtab**)wtr_vector_at(&prog->referenced_symtabs, 0);

		struct wtr_symtab** const end = p_symtab + size;

		while(p_symtab<end)
		{
			struct wtr_symtab* const symtab = *p_symtab++;

			if(0<symtab->refcount)
			{
				--symtab->refcount;
			}
		}
	}
}

void wtr_program_free_metadata(struct wtr_program* const prog)
{
	struct wtr_vm* const vm = prog->vm;

	const size_t size = wtr_vector_size(&prog->metadata_vec);

	if(0<size)
	{

		struct wtr_metadata** ptr = (struct wtr_metadata**)
			wtr_vector_at(&prog->metadata_vec, 0);

		struct wtr_metadata** const end = ptr + size;

		while(ptr<end)
		{
			struct wtr_metadata* const metadata = *ptr++;
			wtr_vm_sys_free(vm, metadata);
		}
	}

	wtr_vector_delete(&prog->metadata_vec, vm);
}

void wtr_program_destroy(struct wtr_vm* const vm,
	struct wtr_program* const program)
{
	struct wtr_symtab* const symtab = program->symtab;

	wtr_program_unref_symtabs(program);

	symtab->destroy(symtab, vm);
	wtr_list_remove(&symtab->link_symtabs);

	wtr_vector_delete(&program->referenced_symtabs, vm);

	wtr_list_remove(&program->link_programs);
	wtr_vm_vma_free(vm, program->program_data);

	memset(program, 0, sizeof(struct wtr_program));
	wtr_vm_sys_free(vm, program);
}
