#ifndef WTRUNTIME_STDLIB_H
#define WTRUNTIME_STDLIB_H

#include "config.h"

/*----------------------------------------------------------------*/
/* exceptions */

#define WTR_STDEXCEPTIONS(FUNCTION) \
	FUNCTION(UnknownException) \
	FUNCTION(BadProgramAccessException) \
	FUNCTION(MemoryAccessViolationException) \
	FUNCTION(StackOverflowException) \
	FUNCTION(UndefinedInstructionException) \
	FUNCTION(OutOfMemoryException) \
	FUNCTION(DivisionByZeroException) \
	FUNCTION(UnhandledControlPathException) \
	FUNCTION(IndexOutOfRangeException) \
	FUNCTION(BadRecordFieldException) \
	FUNCTION(HandleDereferredToNullException) \
	FUNCTION(ConstraintErrorException) \
	FUNCTION(InvalidPFNException) \
	FUNCTION(InvariantViolationException) \
	FUNCTION(PreconditionViolationException) \
	FUNCTION(PostconditionViolationException) \
	FUNCTION(AssertionFailedException) \
	FUNCTION(IOException) \
	FUNCTION(InvocationException) \
	FUNCTION(UnicodeErrorException) \
	FUNCTION(IllegalSequenceException) \
	FUNCTION(EndOfFileException) \


#define __WTR_ENUMERATE_EXCEPTION(exc) exc,

enum
{
	WTR_STDEXCEPTIONS(__WTR_ENUMERATE_EXCEPTION)
};

/*----------------------------------------------------------------*/
/* microprograms */

#define WTR_MICROPROGRAMS(FUNCTION) \
	/*===================== MAIN =====================*/ \
	FUNCTION(undefined,           UNDEFINED          ) \
	/*===================== CONSOLE =====================*/ \
	FUNCTION(conwrite,            CONWRITE           ) \
	FUNCTION(conwritech,          CONWRITECH         ) \
	FUNCTION(conreadln,           CONREADLN          ) \
	/*===================== ARITHMETIC =====================*/ \
	FUNCTION(i8sqrt,              I8SQRT             ) \
	FUNCTION(u8sqrt,              U8SQRT             ) \
	FUNCTION(i16sqrt,             I16SQRT            ) \
	FUNCTION(u16sqrt,             U16SQRT            ) \
	FUNCTION(i32sqrt,             I32SQRT            ) \
	FUNCTION(u32sqrt,             U32SQRT            ) \
	FUNCTION(i64sqrt,             I64SQRT            ) \
	FUNCTION(u64sqrt,             U64SQRT            ) \
	FUNCTION(f32sqrt,             F32SQRT            ) \
	FUNCTION(f64sqrt,             F64SQRT            ) \
	FUNCTION(f32log,              F32LOG             ) \
	FUNCTION(f64log,              F64LOG             ) \
	FUNCTION(f32exp,              F32EXP             ) \
	FUNCTION(f64exp,              F64EXP             ) \
	FUNCTION(f32sin,              F32SIN             ) \
	FUNCTION(f64sin,              F64SIN             ) \
	FUNCTION(f32cos,              F32COS             ) \
	FUNCTION(f64cos,              F64COS             ) \
	FUNCTION(f32tan,              F32TAN             ) \
	FUNCTION(f64tan,              F64TAN             ) \
	FUNCTION(f32asin,             F32ASIN            ) \
	FUNCTION(f64asin,             F64ASIN            ) \
	FUNCTION(f32acos,             F32ACOS            ) \
	FUNCTION(f64acos,             F64ACOS            ) \
	FUNCTION(f32atan,             F32ATAN            ) \
	FUNCTION(f64atan,             F64ATAN            ) \
	FUNCTION(f32atan2,            F32ATAN2           ) \
	FUNCTION(f64atan2,            F64ATAN2           ) \
	FUNCTION(f32pow,              F32POW             ) \
	FUNCTION(f64pow,              F64POW             ) \
	FUNCTION(sarb,                SARB               ) \
	FUNCTION(sarh,                SARH               ) \
	FUNCTION(negb,                NEGB               ) \
	FUNCTION(negh,                NEGH               ) \
	/*===================== BIT OPERATIONS =====================*/ \
	FUNCTION(clz8,                CLZ8               ) \
	FUNCTION(clz16,               CLZ16              ) \
	FUNCTION(clz32,               CLZ32              ) \
	FUNCTION(clz64,               CLZ64              ) \
	FUNCTION(ctz8,                CTZ8               ) \
	FUNCTION(ctz16,               CTZ16              ) \
	FUNCTION(ctz32,               CTZ32              ) \
	FUNCTION(ctz64,               CTZ64              ) \
	/*===================== STRING AND CHAR =====================*/ \
	FUNCTION(strcmp,              STRCMP             ) \
	FUNCTION(stricmp,             STRICMP            ) \
	FUNCTION(strdup,              STRDUP             ) \
	FUNCTION(strcat,              STRCAT             ) \
	FUNCTION(strrev,              STRREV             ) \
	FUNCTION(strsub,              STRSUB             ) \
	FUNCTION(strsub2,             STRSUB2            ) \
	FUNCTION(strlen,              STRLEN             ) \
	FUNCTION(strupper,            STRUPPER           ) \
	FUNCTION(strlower,            STRLOWER           ) \
	FUNCTION(strtrim,             STRTRIM            ) \
	FUNCTION(strtrim2,            STRTRIM2           ) \
	FUNCTION(strempty,            STREMPTY           ) \
	FUNCTION(strstartswith,       STRSTARTSWITH      ) \
	FUNCTION(strendswith,         STRENDSWITH        ) \
	FUNCTION(strchars,            STRCHARS           ) \
	FUNCTION(strchars2,           STRCHARS2          ) \
	FUNCTION(strcopies,           STRCOPIES          ) \
	FUNCTION(strcopies2,          STRCOPIES2         ) \
	FUNCTION(strcpy,              STRCPY             ) \
	FUNCTION(strstr,              STRSTR             ) \
	FUNCTION(strstr2,             STRSTR2            ) \
	FUNCTION(strrstr,             STRRSTR            ) \
	FUNCTION(strrstr2,            STRRSTR2           ) \
	FUNCTION(strchr,              STRCHR             ) \
	FUNCTION(strchr2,             STRCHR2            ) \
	FUNCTION(strrchr,             STRRCHR            ) \
	FUNCTION(strrchr2,            STRRCHR2           ) \
	FUNCTION(strspn,              STRSPN             ) \
	FUNCTION(strspn2,             STRSPN2            ) \
	FUNCTION(strcspn,             STRCSPN            ) \
	FUNCTION(strcspn2,            STRCSPN2           ) \
	FUNCTION(strfromutf8,         STRFROMUTF8        ) \
	FUNCTION(strtoutf8,           STRTOUTF8          ) \
	FUNCTION(chartostring,        CHARTOSTRING       ) \
	FUNCTION(charcmp,             CHARCMP            ) \
	FUNCTION(charadd,             CHARADD            ) \
	FUNCTION(charsub,             CHARSUB            ) \
	FUNCTION(charinc,             CHARINC            ) \
	FUNCTION(chardiff,            CHARDIFF           ) \
	FUNCTION(chartolower,         CHARTOLOWER        ) \
	FUNCTION(chartoupper,         CHARTOUPPER        ) \
	/*===================== SEQ =====================*/ \
	FUNCTION(seqempty,            SEQEMPTY           ) \
	/*===================== CONVERSION =====================*/ \
	FUNCTION(ci32string,          CI32STRING         ) \
	FUNCTION(cu32string,          CU32STRING         ) \
	FUNCTION(ci64string,          CI64STRING         ) \
	FUNCTION(cu64string,          CU64STRING         ) \
	FUNCTION(cf32string,          CF32STRING         ) \
	FUNCTION(cf64string,          CF64STRING         ) \
	FUNCTION(cboolstring,         CBOOLSTRING        ) \
	FUNCTION(cstringi32,          CSTRINGI32         ) \
	FUNCTION(cstringu32,          CSTRINGU32         ) \
	FUNCTION(cstringi64,          CSTRINGI64         ) \
	FUNCTION(cstringu64,          CSTRINGU64         ) \
	FUNCTION(cstringf32,          CSTRINGF32         ) \
	FUNCTION(cstringf64,          CSTRINGF64         ) \
	FUNCTION(cstringbool,         CSTRINGBOOL        ) \
	FUNCTION(ci32u32,             CI32U32            ) \
	FUNCTION(cu32i32,             CU32I32            ) \
	FUNCTION(cu32u64,             CU32U64            ) \
	FUNCTION(cu32i64,             CU32I64            ) \
	FUNCTION(ci32u64,             CI32U64            ) \
	FUNCTION(ci32i64,             CI32I64            ) \
	FUNCTION(cu64u32,             CU64U32            ) \
	FUNCTION(cu64i32,             CU64I32            ) \
	FUNCTION(ci64u32,             CI64U32            ) \
	FUNCTION(ci64i32,             CI64I32            ) \
	FUNCTION(ci64u64,             CI64U64            ) \
	FUNCTION(cu64i64,             CU64I64            ) \
	FUNCTION(cf32i32,             CF32I32            ) \
	FUNCTION(cf32u32,             CF32U32            ) \
	FUNCTION(ci32f32,             CI32F32            ) \
	FUNCTION(cu32f32,             CU32F32            ) \
	FUNCTION(ci32f64,             CI32F64            ) \
	FUNCTION(cu32f64,             CU32F64            ) \
	FUNCTION(cf32f64,             CF32F64            ) \
	FUNCTION(cf64f32,             CF64F32            ) \
	FUNCTION(ci64f32,             CI64F32            ) \
	FUNCTION(cu64f32,             CU64F32            ) \
	FUNCTION(ci64f64,             CI64F64            ) \
	FUNCTION(cu64f64,             CU64F64            ) \
	FUNCTION(cf64i32,             CF64I32            ) \
	FUNCTION(cf64u32,             CF64U32            ) \
	FUNCTION(cf32i64,             CF32I64            ) \
	FUNCTION(cf32u64,             CF32U64            ) \
	FUNCTION(cf64i64,             CF64I64            ) \
	FUNCTION(cf64u64,             CF64U64            ) \
	FUNCTION(cchari32,            CCHARI32           ) \
	FUNCTION(ccharu32,            CCHARU32           ) \
	FUNCTION(cchari64,            CCHARI64           ) \
	FUNCTION(ccharu64,            CCHARU64           ) \
	FUNCTION(ci32char,            CI32CHAR           ) \
	FUNCTION(cu32char,            CU32CHAR           ) \
	FUNCTION(ci64char,            CI64CHAR           ) \
	FUNCTION(cu64char,            CU64CHAR           ) \
	FUNCTION(ci32toi8,            CI32TOI8           ) \
	FUNCTION(ci32tou8,            CI32TOU8           ) \
	FUNCTION(ci32toi16,           CI32TOI16          ) \
	FUNCTION(ci32tou16,           CI32TOU16          ) \
	FUNCTION(cu32toi8,            CU32TOI8           ) \
	FUNCTION(cu32tou8,            CU32TOU8           ) \
	FUNCTION(cu32toi16,           CU32TOI16          ) \
	FUNCTION(cu32tou16,           CU32TOU16          ) \
	FUNCTION(ci64toi8,            CI64TOI8           ) \
	FUNCTION(ci64tou8,            CI64TOU8           ) \
	FUNCTION(ci64toi16,           CI64TOI16          ) \
	FUNCTION(ci64tou16,           CI64TOU16          ) \
	FUNCTION(cu64toi8,            CU64TOI8           ) \
	FUNCTION(cu64tou8,            CU64TOU8           ) \
	FUNCTION(cu64toi16,           CU64TOI16          ) \
	FUNCTION(cu64tou16,           CU64TOU16          ) \
	FUNCTION(ci8tou8,             CI8TOU8            ) \
	FUNCTION(cu8toi8,             CU8TOI8            ) \
	FUNCTION(ci16toi8,            CI16TOI8           ) \
	FUNCTION(ci16tou8,            CI16TOU8           ) \
	FUNCTION(ci16tou16,           CI16TOU16          ) \
	FUNCTION(cu16toi8,            CU16TOI8           ) \
	FUNCTION(cu16tou8,            CU16TOU8           ) \
	FUNCTION(cu16toi16,           CU16TOI16          ) \
	/*===================== HANDLE =====================*/ \
	FUNCTION(nullhandle,          NULLHANDLE         ) \
	FUNCTION(handleequal,         HANDLEEQUAL        ) \
	/*===================== FILE =====================*/ \
	FUNCTION(fileaccess,          FILEACCESS         ) \
	FUNCTION(filestat,            FILESTAT           ) \
	FUNCTION(filestatf,           FILESTATF          ) \
	FUNCTION(fileopen,            FILEOPEN           ) \
	FUNCTION(fileclose,           FILECLOSE          ) \
	FUNCTION(fileeof,             FILEEOF            ) \
	FUNCTION(fileread,            FILEREAD           ) \
	FUNCTION(filereadr,           FILEREADR          ) \
	FUNCTION(filereadstr,         FILEREADSTR        ) \
	FUNCTION(filewrite,           FILEWRITE          ) \
	FUNCTION(filewritestr,        FILEWRITESTR       ) \
	/*===================== DIRECTORY =====================*/ \
	FUNCTION(diropen,             DIROPEN            ) \
	FUNCTION(dirclose,            DIRCLOSE           ) \
	FUNCTION(dirread,             DIRREAD            ) \
	FUNCTION(dirgetname,          DIRGETNAME         ) \
	FUNCTION(dirgettype,          DIRGETTYPE         ) \
	/*===================== SYSTEM =====================*/ \
	FUNCTION(rand,                RAND               ) \
	FUNCTION(randf,               RANDF              ) \
	FUNCTION(srand,               SRAND              ) \
	FUNCTION(islittleendian,      ISLITTLEENDIAN     ) \
	FUNCTION(isbigendian,         ISBIGENDIAN        ) \
	FUNCTION(swap16,              SWAP16             ) \
	FUNCTION(swap32,              SWAP32             ) \
	FUNCTION(swap64,              SWAP64             ) \
	FUNCTION(lswap16,             LSWAP16            ) \
	FUNCTION(lswap32,             LSWAP32            ) \
	FUNCTION(lswap64,             LSWAP64            ) \
	FUNCTION(bswap16,             BSWAP16            ) \
	FUNCTION(bswap32,             BSWAP32            ) \
	FUNCTION(bswap64,             BSWAP64            ) \
	/*===================== CHAR CLASSIFICATION =====================*/ \
	FUNCTION(iscntrl,             ISCNTRL            ) \
	FUNCTION(isprint,             ISPRINT            ) \
	FUNCTION(isspace,             ISSPACE            ) \
	FUNCTION(isblank,             ISBLANK            ) \
	FUNCTION(isgraph,             ISGRAPH            ) \
	FUNCTION(ispunct,             ISPUNCT            ) \
	FUNCTION(isalnum,             ISALNUM            ) \
	FUNCTION(isalpha,             ISALPHA            ) \
	FUNCTION(isupper,             ISUPPER            ) \
	FUNCTION(islower,             ISLOWER            ) \
	FUNCTION(isdigit,             ISDIGIT            ) \
	FUNCTION(isxdigit,            ISXDIGIT           ) \
	/*===================== WELLTYPE RUNTIME =====================*/ \
	FUNCTION(version,             VERSION            ) \
	FUNCTION(compversion,         COMPVERSION        ) \
	FUNCTION(coreversion,         COREVERSION        ) \

#define __WTR_ENUMERATE_MICROPROGRAM(lowercase,UPPERCASE) WTMP_##UPPERCASE ,

enum
{
	WTR_MICROPROGRAMS(__WTR_ENUMERATE_MICROPROGRAM)
	WTR_NUM_MICROPROGRAMS
};

#endif /* WTRUNTIME_STDLIB_H */
